######################################################################
#
# generate_bread_crumb( $session )
#
######################################################################
#
# $session 
# - Session object (the current session)
#
# returns: $bcUL
# - <ul> DOM
#
######################################################################
#
# Called by dynamic_template.pl, generate the bread crumb on each page.
# The breadcrumb follows such manner: <ul id="breadcrumbs" /> -> <li /> -> <a href="" /> -> Text
#
# Author: Bin Han and Tomasz Neugebauer
#
######################################################################

$c->{generate_bread_crumb} = sub
{
	my ($session) = @_;
	
	my @URL = ();
	my $URLlength = 0;

	#Get the uri of the page.
	if (defined($session->get_request))
	{
		@URL = split(/\//, $session->get_request->uri);
		$URLlength = @URL;
	}
	
	#initial the bread crumb unordered list
	my $bcUL = $session->make_element("ul", id=>"breadcrumbs");
	my $bcLI = $session->make_element("li");
	my $bcURL = $session->get_repository->get_conf( "base_url" );
	my $bcA = $session->make_element("a", href=>"$bcURL");
	$bcA->appendChild($session->make_text("Home"));
  	$bcLI->appendChild($bcA);
	#append the home <li> element to <ul>
  	$bcUL->appendChild($bcLI);
	
	#Information for the static pages.
	my %static_pages = (
		'information.html' => "About",
		'policies.html' => "Policy",
		'faq.html' => "FAQ",
		'contact.html' => "Contact",
	);
	
	#Hash of arrays, store the structure of the static pages.
	my %static_pages_structure = (
		'information.html' => ['information.html'],
		'policies.html' => ['information.html', 'policies.html'],
		'faq.html' => ['information.html', 'faq.html'],
		'contact.html' => ['information.html', 'contact.html'],
	);
	
	#Information for the cgi pages.
	my %cgi_pages = (
		'latest' => "Latest Additions",
		'search' => "Search",
	);

	#research repository Home
	if ($URLlength == 0)
	{
		return undef;
	}
	#static pages
	elsif ($URL[1] && defined $static_pages{$URL[1]})
	{
		render_staticpage_crumb($session, $bcURL, $URL[1], $bcUL, \%static_pages, \%static_pages_structure);
	}
	#cgi pages
	elsif ($URL[1] eq "cgi" && defined $cgi_pages{$URL[2]})
	{
		render_cgi_crumb($session, $bcURL, $URL[2], $bcUL, \%cgi_pages);
	}
	#abstract pages
  	elsif ($URL[1] =~ /(\d+)/)	#research repository -> abstracts. the bread crumb will be: Home > Title
	{
		# $1 is the abstract (EPrint) id get from $URL[1] =~ /(\d+)/.
		my $abstract = EPrints::DataObj::EPrint->new( $session, $1 );
		my $abstractTitle = $abstract->get_value("title");
		render_crumb_li_withSeparator($session, $abstract->get_url(), $abstractTitle, ">", \$bcUL );
		return $bcUL;
	}
	#/view/year/*
  	elsif ($URL[1] eq "view" and $URL[2] eq "year") #research repository -> Browse by Year
  	{
    	my $currentURL = $bcURL."/".$URL[1]."/".$URL[2];
    	render_crumb_li_withSeparator($session, $currentURL, "Browse by Year", ">", \$bcUL );
    	
    	if ($URL[3])
    	{
      		$_ = $URL[3];
			s/.html//;
      		$currentURL = $currentURL."/".$URL[3];
			# $_ is the year (e.g. 2009) get from s/.html//.
			render_crumb_li_withSeparator($session, $currentURL, ("Year: ".$_), ">", \$bcUL );
    	}
    	
		return $bcUL;
  	}
	#/view/divisions/*
  	elsif ($URL[1] eq "view" and $URL[2] eq "divisions")  #research repository -> Browse by Department
	{
		#add crumb "> Browse by Department"
    	my $currentURL = $bcURL."/view/".$URL[2];

		render_crumb_li_withSeparator($session, $currentURL, "Browse by Department", ">", \$bcUL );
    	
		#/view/divisions/*
	    if ($URL[3])	
	    { 
			#research repository -> Browse by Department -> Faculties and Departments
	    	if ($URL[3] eq "divisions")
	      	{
	        	$currentURL = $currentURL."/".$URL[3];
				render_crumb_li_withSeparator($session, $currentURL, "Faculties and Departments", ">", \$bcUL );
	      	}
			#research repository -> Browse by Department -> Concordia University
	      	elsif ($URL[3] eq "concordia")
	      	{
	        	$currentURL = $currentURL."/".$URL[3];
				render_crumb_li_withSeparator($session, $currentURL, "Concordia University", ">", \$bcUL );
				
				#$URL[4] is the value for year, here assuming that in view.pl divisions section: fields => "divisions,-date;res=year",
				#/view/divisions/subjectid/year
				if ($URL[4])
				{
					$_ = $URL[4];
					s/.html//;
					$currentURL = $currentURL."/".$URL[4];
					# $_ is the year (e.g. 2009) get from s/.html//.
					render_crumb_li_withSeparator($session, $currentURL, ("Year: ".$_), ">", \$bcUL );
				}
	      	}
			#research repository -> Browse by Department -> Concordia University -> Faculty and Department
	     	else
	      	{
				render_crumb_divisions(\$session, \$bcUL, \$currentURL, \$URL[3]);
				
				#Add crumb for year	        	
	        	if ($URL[4])
				{
					$_ = $URL[4];
					s/.html//;
					$currentURL = $currentURL."/".$URL[4];
					# $_ is the year (e.g. 2009) get from s/.html//.
					render_crumb_li_withSeparator($session, $currentURL, ("Year: ".$_), ">", \$bcUL );
				}
			}
		}

		return $bcUL;
	}
	#/view/creators/*
	elsif ($URL[1] eq "view" and $URL[2] eq "creators") #research repository -> Browse by Author
	{
		my $currentURL = $bcURL."/".$URL[1]."/".$URL[2];
		render_crumb_li_withSeparator($session, $currentURL, "Browse by Author", ">", \$bcUL );
		    		
    	if ($URL[3])
    	{
	    	$_ = $URL[3];
	    	if (/^index\.(\w+)\.html$/)	#crumb for A...
	    	{
		    	$currentURL = $currentURL."/".$URL[3];
				# $1 get from /^index\.(\w+)\.html$/ indicates the first letter of the creator's name.
				render_crumb_li_withSeparator($session, $currentURL, ($1."..."), ">", \$bcUL );
		    }
		    elsif (/(.+)=(.+)=(.+)=/)	#crumb for Author name
		    {
			    #Add crumb "\W..."
			    my $parentURL = $currentURL."/index.".substr($_, 0, 1).".html";
				# substr($_, 0, 1) get the first letter of the creator's name.
				render_crumb_li_withSeparator($session, $parentURL, (substr($_, 0, 1)."..."), ">", \$bcUL );
				
				#Add author name, eliminate the .html suffix.
		    	/^(.+)\./;
				$currentURL = $currentURL."/".$URL[3];
		    	my $author = EPrints::Utils::unescape_filename($1);
		    	# FirstName:LastName:: -> FirstName, LastName
		    	my @name = split(/:/, $author);
				
				render_crumb_li_withSeparator($session, $currentURL, ($name[0].", ".$name[1]), ">", \$bcUL );
			}
	    }
    	
		return $bcUL;
	}
	#/view/doctype/*
	elsif ($URL[1] eq "view" and $URL[2] eq "doctype") #research repository -> Browse by Document Type
	{
		my $currentURL = $bcURL."/".$URL[1]."/".$URL[2];
		render_crumb_li_withSeparator($session, $currentURL, "Browse by Document Type", ">", \$bcUL );
    	
    	if ($URL[3])
    	{
	    	$_ = $URL[3];
	    	# Eliminate the .html suffix.
			/^(.+)\./;
			$currentURL = $currentURL."/".$URL[3];

	    	my $doctype = EPrints::Utils::unescape_filename($1);			
			# Remove '_'
			my @doctype_name_split = split(/_/, $doctype);
			# Capitalize the first letter
			my @Doctype_Name_Split = map(ucfirst, @doctype_name_split);
			# Add space
			my $doctype_name_join = join(' ', @Doctype_Name_Split);
			
			render_crumb_li_withSeparator($session, $currentURL, $doctype_name_join, ">", \$bcUL );
	    }
    	
		return $bcUL;
	}
	else
	{
		return $bcUL;
	}
	
	#Render <li> element for the breadcrumb
	#$url: url of the breadcrumb, $name: name appeared of the breadcrumb
	sub render_crumb_li
	{
		my ($session, $url, $name) = @_;
		my $crumbA = $session->make_element("a", href=>"$url");
		$crumbA->appendChild($session->make_text( $name ));
		my $crumbLI = $session->make_element("li");
		$crumbLI->appendChild($crumbA);
		return $crumbLI;
	}
	#Render the <li> separator
	#the seperator to use in the breadcrumb(>)
	sub render_crumb_li_separator
	{
		my ($session, $separator) = @_;
		my $separatorLI = $session->make_element("li");
		$separatorLI->appendChild($session->make_text( $separator ));
		return $separatorLI;
	}
	
	#Append two <li> elements to <ul>, a crumb and the separetor, pass by reference
	#render_crumb_li_withSeparator($session, $url, $name, $separator, \$bcUL)
	#$ref_bcUL: pass in the reference to the whole breadcrumb UL tag.
	sub render_crumb_li_withSeparator
	{
		my ($session, $url, $name, $separator, $ref_bcUL) = @_;
		${$ref_bcUL}->appendChild( &render_crumb_li_separator($session, $separator) );
		${$ref_bcUL}->appendChild( &render_crumb_li($session, $url, $name) );
	}
	
	#Render the division breadcrumb, e.g. Concordia University>Faculty of Arts and Science>Communication Studies
	#Pass by reference, render_crumb_divisions(\$session, \$bcUL, \$currentURL, \$URL[3]);
	#$ref_bcUL: reference to the breadcrumb ul tag; $ref_currentURL: reference to the url of current page; $ref_URI: the urn to the current page, with which to produce the subjectid.
	sub render_crumb_divisions
	{
		my $ref_session = shift;
		my $ref_bcUL = shift;
		my $ref_currentURL = shift;
		my $ref_URI = shift;
		
		# An array of hash, contains the division route.
		my @divisions;
		
		#Add Concordia U first
		${$ref_bcUL}->appendChild( &render_crumb_li_separator(${$ref_session}, ">") );
		${$ref_bcUL}->appendChild( &render_crumb_li(${$ref_session}, (${$ref_currentURL}."/concordia/"), "Concordia University") ); 
			
		#Decode the uri $URL[3] to subjectid, and create the subject object
		my $subjectid = EPrints::Utils::unescape_filename(${$ref_URI});
		my $subject = EPrints::DataObj::Subject->new(${$ref_session}, $subjectid);
		my $url_flag = ${$ref_currentURL}."/".EPrints::Utils::escape_filename($subjectid) if ( (defined $subject) && ($subjectid ne "concordia") );
		
		while ( (defined $subject) && ($subjectid ne "concordia") )
		{
			#Get subject name of a given subject object
			my $subjectname = EPrints::Utils::tree_to_utf8( $subject->render_description() );
			my $subjecturl =  ${$ref_currentURL}."/".EPrints::Utils::escape_filename($subjectid);
			
			push @divisions, { subject_name=>$subjectname, subject_url=>$subjecturl };
			
			#Get the parents subject of a given subject object
			my @parent = $subject->get_parents();
			$subject = pop @parent;
			#Get subjectid from a given subject object
			$subjectid = $subject->{data}->{subjectid};
		}
		
		while (@divisions)
		{	
			my $division = pop @divisions;
			${$ref_bcUL}->appendChild( &render_crumb_li_separator(${$ref_session}, ">") );
			${$ref_bcUL}->appendChild( &render_crumb_li( ${$ref_session}, $division->{"subject_url"}, $division->{"subject_name"} ) ); 	
		}
		
		${$ref_currentURL} = $url_flag;
	}
	
	#Render static page crumb.
	#$bcURL: the url for the current page; $urn: the urn for current page (faq.html); $bcUL: the breadcrumb ul tag;
	#$ref_static_pages: reference to %static_pages; $ref_static_pages_structure: reference to %static_pages_structure.
	sub render_staticpage_crumb
	{
		my ($session, $bcURL, $urn, $bcUL, $ref_static_pages, $ref_static_pages_structure) = @_;
		
		if(defined ${$ref_static_pages}{$urn})
		{	
			#get the value which is an array with the path for the static page by key($urn) from %static_pages_structure.
			my @page_structure = @{${$ref_static_pages_structure}{$urn}} if defined ${$ref_static_pages_structure}{$urn};
			#loop the array and render crumb for each item in the array.
			foreach my $page (@page_structure)
			{
				my $currentURL = $bcURL."/".$page;
				render_crumb_li_withSeparator($session, $currentURL, ${$ref_static_pages}{$page}, ">", \$bcUL );
			}
		}
		return $bcUL;
	}
	
	#Render the cgi/* crumb,
	#$ref_cgi_pages: reference to %cgi_pages.
	sub render_cgi_crumb
	{
		my ($session, $bcURL, $URN, $bcUL, $ref_cgi_pages) = @_;
		
		my $currentURL = $bcURL."/cgi/".$URN;
		render_crumb_li_withSeparator($session, $currentURL, ${$ref_cgi_pages}{$URN}, ">", \$bcUL);
		return $bcUL;
	}
};